@@ -4,25 +4,31 @@
 """Extension manager using pip as package manager and PyPi.org as packages source."""
 
 import asyncio
+import http.client
 import io
 import json
 import math
 import re
 import sys
+import tempfile
 import xmlrpc.client
 from datetime import datetime, timedelta, timezone
 from functools import partial
 from itertools import groupby
+from os import environ
 from pathlib import Path
 from subprocess import CalledProcessError, run
 from tarfile import TarFile
 from typing import Any, Callable, Dict, List, Optional, Tuple
+from urllib.parse import urlparse
 from zipfile import ZipFile
 
+import httpx
 import tornado
 from async_lru import alru_cache
 from traitlets import CFloat, CInt, Unicode, config, observe
 
+from jupyterlab._version import __version__
 from jupyterlab.extensions.manager import (
     ActionResult,
     ExtensionManager,
@@ -31,29 +37,72 @@ from jupyterlab.extensions.manager import (
 )
 
 
-async def _fetch_package_metadata(name: str, latest_version: str, base_url: str) -> dict:
-    http_client = tornado.httpclient.AsyncHTTPClient()
-    response = await http_client.fetch(
+class ProxiedTransport(xmlrpc.client.Transport):
+    def set_proxy(self, host, port=None, headers=None):
+        self.proxy = host, port
+        self.proxy_headers = headers
+
+    def make_connection(self, host):
+        connection = http.client.HTTPConnection(*self.proxy)
+        connection.set_tunnel(host, headers=self.proxy_headers)
+        self._connection = host, connection
+        return connection
+
+
+xmlrpc_transport_override = None
+
+all_proxy_url = environ.get("ALL_PROXY")
+# For historical reasons, we also support the lowercase environment variables.
+# Info: https://about.gitlab.com/blog/2021/01/27/we-need-to-talk-no-proxy/
+http_proxy_url = environ.get("http_proxy") or environ.get("HTTP_PROXY") or all_proxy_url
+https_proxy_url = (
+    environ.get("https_proxy") or environ.get("HTTPS_PROXY") or http_proxy_url or all_proxy_url
+)
+proxies = None
+
+if http_proxy_url:
+    http_proxy = urlparse(http_proxy_url)
+    proxy_host, _, proxy_port = http_proxy.netloc.partition(":")
+
+    proxies = {
+        "http://": http_proxy_url,
+        "https://": https_proxy_url,
+    }
+
+    xmlrpc_transport_override = ProxiedTransport()
+    xmlrpc_transport_override.set_proxy(proxy_host, proxy_port)
+
+
+async def _fetch_package_metadata(
+    client: httpx.AsyncClient,
+    name: str,
+    latest_version: str,
+    base_url: str,
+) -> dict:
+    response = await client.get(
         base_url + f"/{name}/{latest_version}/json",
         headers={"Content-Type": "application/json"},
     )
-    data = json.loads(response.body).get("info")
+    if response.status_code < 400:  # noqa PLR2004
+        data = json.loads(response.text).get("info")
 
-    # Keep minimal information to limit cache size
-    return {
-        k: data.get(k)
-        for k in [
-            "author",
-            "bugtrack_url",
-            "docs_url",
-            "home_page",
-            "license",
-            "package_url",
-            "project_url",
-            "project_urls",
-            "summary",
-        ]
-    }
+        # Keep minimal information to limit cache size
+        return {
+            k: data.get(k)
+            for k in [
+                "author",
+                "bugtrack_url",
+                "docs_url",
+                "home_page",
+                "license",
+                "package_url",
+                "project_url",
+                "project_urls",
+                "summary",
+            ]
+        }
+    else:
+        return {}
 
 
 class PyPIExtensionManager(ExtensionManager):
@@ -82,18 +131,24 @@ class PyPIExtensionManager(ExtensionManager):
         parent: Optional[config.Configurable] = None,
     ) -> None:
         super().__init__(app_options, ext_options, parent)
+        self._httpx_client = httpx.AsyncClient(proxies=proxies)
         # Set configurable cache size to fetch function
-        self._fetch_package_metadata = _fetch_package_metadata
+        self._fetch_package_metadata = partial(_fetch_package_metadata, self._httpx_client)
         self._observe_package_metadata_cache_size({"new": self.package_metadata_cache_size})
         # Combine XML RPC API and JSON API to reduce throttling by PyPI.org
-        self._http_client = tornado.httpclient.AsyncHTTPClient()
-        self._rpc_client = xmlrpc.client.ServerProxy(self.base_url)
+        self._rpc_client = xmlrpc.client.ServerProxy(
+            self.base_url, transport=xmlrpc_transport_override
+        )
         self.__last_all_packages_request_time = datetime.now(tz=timezone.utc) - timedelta(
             seconds=self.cache_timeout * 1.01
         )
         self.__all_packages_cache = None
 
         self.log.debug(f"Extensions list will be fetched from {self.base_url}.")
+        if xmlrpc_transport_override:
+            self.log.info(
+                f"Extensions will be fetched using proxy, proxy host and port: {xmlrpc_transport_override.proxy}"
+            )
 
     @property
     def metadata(self) -> ExtensionManagerMetadata:
@@ -109,16 +164,19 @@ class PyPIExtensionManager(ExtensionManager):
             The latest available version
         """
         try:
-            http_client = tornado.httpclient.AsyncHTTPClient()
-            response = await http_client.fetch(
-                self.base_url + f"/{pkg}/json",
-                headers={"Content-Type": "application/json"},
+            response = await self._httpx_client.get(
+                self.base_url + f"/{pkg}/json", headers={"Content-Type": "application/json"}
             )
-            data = json.loads(response.body).get("info")
+
+            if response.status_code < 400:  # noqa PLR2004
+                data = json.loads(response.content).get("info", {})
+            else:
+                self.log.debug(f"Failed to get package information on PyPI; {response!s}")
+                return None
         except Exception:
             return None
         else:
-            return ExtensionManager.get_semver_version(data.get("version"))
+            return ExtensionManager.get_semver_version(data.get("version", "")) or None
 
     def get_normalized_name(self, extension: ExtensionPackage) -> str:
         """Normalize extension name.
@@ -175,7 +233,9 @@ class PyPIExtensionManager(ExtensionManager):
 
     @observe("package_metadata_cache_size")
     def _observe_package_metadata_cache_size(self, change):
-        self._fetch_package_metadata = alru_cache(maxsize=change["new"])(_fetch_package_metadata)
+        self._fetch_package_metadata = alru_cache(maxsize=change["new"])(
+            partial(_fetch_package_metadata, self._httpx_client)
+        )
 
     async def list_packages(
         self, query: str, page: int, per_page: int
@@ -276,100 +336,110 @@ class PyPIExtensionManager(ExtensionManager):
             The action result
         """
         current_loop = tornado.ioloop.IOLoop.current()
+        with tempfile.TemporaryDirectory() as ve_dir, tempfile.NamedTemporaryFile(
+            mode="w+", dir=ve_dir, delete=False
+        ) as fconstraint:
+            fconstraint.write(f"jupyterlab=={__version__}")
+            fconstraint.flush()
 
-        cmdline = [
-            sys.executable,
-            "-m",
-            "pip",
-            "install",
-            "--no-input",
-            "--quiet",
-            "--progress-bar",
-            "off",
-        ]
-        if version is not None:
-            cmdline.append(f"{name}=={version}")
-        else:
-            cmdline.append(name)
+            cmdline = [
+                sys.executable,
+                "-m",
+                "pip",
+                "install",
+                "--no-input",
+                "--quiet",
+                "--progress-bar",
+                "off",
+                "--constraint",
+                fconstraint.name,
+            ]
+            if version is not None:
+                cmdline.append(f"{name}=={version}")
+            else:
+                cmdline.append(name)
+
+            pkg_action = {}
+            try:
+                tmp_cmd = cmdline.copy()
+                tmp_cmd.insert(-1, "--dry-run")
+                tmp_cmd.insert(-1, "--report")
+                tmp_cmd.insert(-1, "-")
+                result = await current_loop.run_in_executor(
+                    None, partial(run, tmp_cmd, capture_output=True, check=True)
+                )
+
+                action_info = json.loads(result.stdout.decode("utf-8"))
+                pkg_action = next(
+                    filter(
+                        lambda p: p.get("metadata", {}).get("name") == name.replace("_", "-"),
+                        action_info.get("install", []),
+                    )
+                )
+            except CalledProcessError as e:
+                self.log.debug(f"Fail to get installation report: {e.stderr}", exc_info=e)
+            except Exception as err:
+                self.log.debug("Fail to get installation report.", exc_info=err)
+            else:
+                self.log.debug(f"Actions to be executed by pip {json.dumps(action_info)}.")
+
+            self.log.debug(f"Executing '{' '.join(cmdline)}'")
 
-        pkg_action = {}
-        try:
-            tmp_cmd = cmdline.copy()
-            tmp_cmd.insert(-1, "--dry-run")
-            tmp_cmd.insert(-1, "--report")
-            tmp_cmd.insert(-1, "-")
             result = await current_loop.run_in_executor(
-                None, partial(run, tmp_cmd, capture_output=True, check=True)
+                None, partial(run, cmdline, capture_output=True)
             )
 
-            action_info = json.loads(result.stdout.decode("utf-8"))
-            pkg_action = list(
-                filter(
-                    lambda p: p.get("metadata", {}).get("name") == name.replace("_", "-"),
-                    action_info.get("install", []),
-                )
-            )[0]
-        except CalledProcessError as e:
-            self.log.debug(f"Fail to get installation report: {e.stderr}", exc_info=e)
-        except Exception as err:
-            self.log.debug("Fail to get installation report.", exc_info=err)
-        else:
-            self.log.debug(f"Actions to be executed by pip {json.dumps(action_info)}.")
+            self.log.debug(f"return code: {result.returncode}")
+            self.log.debug(f"stdout: {result.stdout.decode('utf-8')}")
+            error = result.stderr.decode("utf-8")
+            if result.returncode == 0:
+                self.log.debug(f"stderr: {error}")
+                # Figure out if the package has server or kernel parts
+                jlab_metadata = None
+                try:
+                    download_url: str = pkg_action.get("download_info", {}).get("url")
+                    if download_url is not None:
+                        response = await self._httpx_client.get(download_url)
+                        if response.status_code < 400:  # noqa PLR2004
+                            if download_url.endswith(".whl"):
+                                with ZipFile(io.BytesIO(response.content)) as wheel:
+                                    for name in filter(
+                                        lambda f: Path(f).name == "package.json",
+                                        wheel.namelist(),
+                                    ):
+                                        data = json.loads(wheel.read(name))
+                                        jlab_metadata = data.get("jupyterlab")
+                                        if jlab_metadata is not None:
+                                            break
+                            elif download_url.endswith("tar.gz"):
+                                with TarFile(io.BytesIO(response.content)) as sdist:
+                                    for name in filter(
+                                        lambda f: Path(f).name == "package.json",
+                                        sdist.getnames(),
+                                    ):
+                                        data = json.load(sdist.extractfile(sdist.getmember(name)))
+                                        jlab_metadata = data.get("jupyterlab")
+                                        if jlab_metadata is not None:
+                                            break
+                        else:
+                            self.log.debug(f"Failed to get '{download_url}'; {response!s}")
+                except Exception as e:
+                    self.log.debug("Fail to get package.json.", exc_info=e)
 
-        self.log.debug(f"Executing '{' '.join(cmdline)}'")
+                follow_ups = [
+                    "frontend",
+                ]
+                if jlab_metadata is not None:
+                    discovery = jlab_metadata.get("discovery", {})
+                    if "kernel" in discovery:
+                        follow_ups.append("kernel")
+                    if "server" in discovery:
+                        follow_ups.append("server")
 
-        result = await current_loop.run_in_executor(
-            None, partial(run, cmdline, capture_output=True)
-        )
-
-        self.log.debug(f"return code: {result.returncode}")
-        self.log.debug(f"stdout: {result.stdout.decode('utf-8')}")
-        error = result.stderr.decode("utf-8")
-        if result.returncode == 0:
-            self.log.debug(f"stderr: {error}")
-            # Figure out if the package has server or kernel parts
-            jlab_metadata = None
-            try:
-                download_url: str = pkg_action.get("download_info", {}).get("url")
-                if download_url is not None:
-                    response = await self._http_client.fetch(download_url)
-                    if download_url.endswith(".whl"):
-                        with ZipFile(io.BytesIO(response.body)) as wheel:
-                            for name in filter(
-                                lambda f: Path(f).name == "package.json",
-                                wheel.namelist(),
-                            ):
-                                data = json.loads(wheel.read(name))
-                                jlab_metadata = data.get("jupyterlab")
-                                if jlab_metadata is not None:
-                                    break
-                    elif download_url.endswith("tar.gz"):
-                        with TarFile(io.BytesIO(response.body)) as sdist:
-                            for name in filter(
-                                lambda f: Path(f).name == "package.json",
-                                sdist.getnames(),
-                            ):
-                                data = json.load(sdist.extractfile(sdist.getmember(name)))
-                                jlab_metadata = data.get("jupyterlab")
-                                if jlab_metadata is not None:
-                                    break
-            except Exception as e:
-                self.log.debug("Fail to get package.json.", exc_info=e)
-
-            follow_ups = [
-                "frontend",
-            ]
-            if jlab_metadata is not None:
-                discovery = jlab_metadata.get("discovery", {})
-                if "kernel" in discovery:
-                    follow_ups.append("kernel")
-                if "server" in discovery:
-                    follow_ups.append("server")
-
-            return ActionResult(status="ok", needs_restart=follow_ups)
-        else:
-            self.log.error(f"Failed to installed {name}: code {result.returncode}\n{error}")
-            return ActionResult(status="error", message=error)
+                return ActionResult(status="ok", needs_restart=follow_ups)
+            else:
+                self.log.error(f"Failed to installed {name}: code {result.returncode}\n{error}")
+                return ActionResult(status="error", message=error)
 
     async def uninstall(self, extension: str) -> ActionResult:
         """Uninstall the required extension.
