@@ -11,11 +11,10 @@ from jupyter_server.extension.handler import ExtensionHandlerMixin
 from tornado import gen, web
 from tornado.concurrent import run_on_executor
 
-from ..commands import AppOptions, _ensure_options, build, build_check, clean
-from ..coreconfig import CoreConfig
+from jupyterlab.commands import AppOptions, _ensure_options, build, build_check, clean
 
 
-class Builder(object):
+class Builder:
     building = False
     executor = ThreadPoolExecutor(max_workers=5)
     canceled = False
@@ -34,9 +33,9 @@ class Builder(object):
     @gen.coroutine
     def get_status(self):
         if self.core_mode:
-            raise gen.Return(dict(status="stable", message=""))
+            raise gen.Return({"status": "stable", "message": ""})
         if self.building:
-            raise gen.Return(dict(status="building", message=""))
+            raise gen.Return({"status": "building", "message": ""})
 
         try:
             messages = yield self._run_build_check(
@@ -44,21 +43,22 @@ class Builder(object):
             )
             status = "needed" if messages else "stable"
             if messages:
-                self.log.warn("Build recommended")
-                [self.log.warn(m) for m in messages]
+                self.log.warning("Build recommended")
+                [self.log.warning(m) for m in messages]
             else:
                 self.log.info("Build is up to date")
-        except ValueError as e:
-            self.log.warn("Could not determine jupyterlab build status without nodejs")
+        except ValueError:
+            self.log.warning("Could not determine jupyterlab build status without nodejs")
             status = "stable"
             messages = []
 
-        raise gen.Return(dict(status=status, message="\n".join(messages)))
+        raise gen.Return({"status": status, "message": "\n".join(messages)})
 
     @gen.coroutine
     def build(self):
         if self._canceling:
-            raise ValueError("Cancel in progress")
+            msg = "Cancel in progress"
+            raise ValueError(msg)
         if not self.building:
             self.canceled = False
             self._future = future = gen.Future()
@@ -84,7 +84,8 @@ class Builder(object):
     @gen.coroutine
     def cancel(self):
         if not self.building:
-            raise ValueError("No current build")
+            msg = "No current build"
+            raise ValueError(msg)
         self._canceling = True
         yield self._future
         self._canceling = False
@@ -112,17 +113,17 @@ class Builder(object):
         )
         try:
             return build(app_options=app_options)
-        except Exception as e:
+        except Exception:
             if self._kill_event.is_set():
                 return
-            self.log.warn("Build failed, running a clean and rebuild")
+            self.log.warning("Build failed, running a clean and rebuild")
             clean(app_options=app_options)
             return build(app_options=app_options)
 
 
 class BuildHandler(ExtensionHandlerMixin, APIHandler):
     def initialize(self, builder=None, name=None):
-        super(BuildHandler, self).initialize(name=name)
+        super().initialize(name=name)
         self.builder = builder
 
     @web.authenticated
@@ -138,7 +139,7 @@ class BuildHandler(ExtensionHandlerMixin, APIHandler):
         try:
             yield self.builder.cancel()
         except Exception as e:
-            raise web.HTTPError(500, str(e))
+            raise web.HTTPError(500, str(e)) from None
         self.set_status(204)
 
     @web.authenticated
@@ -148,7 +149,7 @@ class BuildHandler(ExtensionHandlerMixin, APIHandler):
         try:
             yield self.builder.build()
         except Exception as e:
-            raise web.HTTPError(500, str(e))
+            raise web.HTTPError(500, str(e)) from None
 
         if self.builder.canceled:
             raise web.HTTPError(400, "Build canceled")
