@@ -312,9 +312,9 @@ src[STAR] = "(<|>)?=?\\s*\\*"
 
 # version name recovery for convenient
 RECOVERYVERSIONNAME = R()
-src[RECOVERYVERSIONNAME] = "v?({n})(?:\\.({n}))?{pre}?".format(
-    n=src[NUMERICIDENTIFIER], pre=src[PRERELEASELOOSE]
-)
+_n = src[NUMERICIDENTIFIER]
+_pre = src[PRERELEASELOOSE]
+src[RECOVERYVERSIONNAME] = f"v?({_n})(?:\\.({_n}))?{_pre}?"
 
 #  Compile to actual regexp objects.
 #  All are flag-free, unless they were created above with a flag.
@@ -410,7 +410,7 @@ class SemVer:
 
         self.format()  # xxx:
 
-    def format(self):  # noqa
+    def format(self):
         self.version = f"{self.major}.{self.minor}.{self.patch}"
         if len(self.prerelease) > 0:
             self.version += "-{}".format(".".join(str(v) for v in self.prerelease))
@@ -761,7 +761,7 @@ class Range:
     def __repr__(self):
         return f'<SemVer Range "{self.range}">'
 
-    def format(self):  # noqa
+    def format(self):
         self.range = "||".join(
             [" ".join(c.value for c in comps).strip() for comps in self.set]
         ).strip()
@@ -954,7 +954,7 @@ def replace_caret(comp, loose):
             else:
                 ret = ">=" + M + "." + m + "." + (p or "") + pr + " <" + str(int(M) + 1) + ".0.0"
         else:
-            if M == "0":  # noqa PLR5501
+            if M == "0":
                 if m == "0":
                     ret = (
                         ">="
